'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash.isfunction');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.clone');

var _lodash4 = _interopRequireDefault(_lodash3);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } /**
                                                                                                                                                                                                     * The top-level merge function
                                                                                                                                                                                                     *
                                                                                                                                                                                                     * @param {object} remerge map
                                                                                                                                                                                                     * @return {function} reducer function
                                                                                                                                                                                                     *
                                                                                                                                                                                                     */

var collectionRegex = /^\$(.+)/;
var legacyRegex = /^__(.+)__$/;
var remergeLegacyKey = '$legacy';

var merge = function merge(schema) {
  var debugMode = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];


  var _getAccessorKey = function _getAccessorKey(key) {
    // this regex tests if the key is of the form $abcd1234
    var captureAccessor = collectionRegex.exec(key);
    if (captureAccessor) {
      return captureAccessor[1];
    } else {
      return null;
    }
  };

  var _getLegacyKey = function _getLegacyKey(key) {
    // this regex tests if the key is of the form __abcd__
    var captureLegacy = legacyRegex.exec(key);
    if (captureLegacy) {
      return captureLegacy[1];
    } else {
      return null;
    }
  };

  var _preprocess = function _preprocess(schema) {
    var map = new Map();
    function _dive(subSchema) {
      var prefix = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];
      var path = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];
      var params = arguments.length <= 3 || arguments[3] === undefined ? [] : arguments[3];

      for (var key in subSchema) {
        if (key === '_') {
          continue;
        }
        var child = subSchema[key];
        var type = '' + prefix + key;

        if ((0, _lodash2.default)(child)) {
          var legacyKey = _getLegacyKey(key);
          if (legacyKey) {
            var node = {
              reducer: child,
              path: [legacyKey]
            };

            var nodes = map.get(remergeLegacyKey) || [];
            map.set(remergeLegacyKey, [].concat(_toConsumableArray(nodes), [node]));
          } else {
            var node = {
              reducer: child,
              path: path,
              params: params
            };

            var nodes = map.get(type) || [];
            map.set(type, [].concat(_toConsumableArray(nodes), [node]));
          }
        } else {
          var param = _getAccessorKey(key);
          if (param) {
            _dive(child, prefix, [].concat(_toConsumableArray(path), [key]), [].concat(_toConsumableArray(params), [param]));
          } else {
            _dive(child, type + '.', [].concat(_toConsumableArray(path), [key]), params);
          }
        }
      }
    }

    _dive(schema);
    return map;
  };

  var _initial = function _initial(map) {
    if (!map) {
      return undefined;
    } else if ((0, _lodash2.default)(map)) {
      return undefined;
    }

    var newMap = map['_'] || {};
    var changed = map['_'] !== undefined;

    for (var key in map) {
      if (key === '_') {
        continue;
      }
      if (_getLegacyKey(key)) {
        newMap[_getLegacyKey(key)] = map[key](undefined, {});
        changed = true;
      }

      // to avoid adding keys to collections
      if (!_getAccessorKey(key)) {
        var result = _initial(map[key]);
        if (result !== undefined) {
          newMap[key] = result;
          changed = true;
        }
      }
    }

    return changed ? newMap : null;
  };

  var _reduce = function _reduce(state, action, node) {
    var newState = (0, _lodash4.default)(state);
    var current = newState;
    var parent = null;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = node.path[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var key = _step.value;

        var _accessorKey = _getAccessorKey(key);
        if (_accessorKey) {
          parent = current;
          current = (0, _lodash4.default)((0, _utils.getCollectionElement)(current, action[_accessorKey]));
          (0, _utils.setCollectionElement)(parent, action[_accessorKey], current);
        } else {
          parent = current;
          current = (0, _lodash4.default)((0, _utils.getCollectionElement)(current, key));
          (0, _utils.setCollectionElement)(parent, key, current);
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    current = node.reducer(current, action);

    var lastKey = node.path[node.path.length - 1];
    var accessorKey = _getAccessorKey(lastKey);
    if (accessorKey) {
      (0, _utils.setCollectionElement)(parent, action[accessorKey], current);
    } else {
      (0, _utils.setCollectionElement)(parent, lastKey, current);
    }
    return newState;
  };

  var initialState = _initial(schema);
  var map = _preprocess(schema);

  return function (state, action) {
    if (action === undefined) {
      (0, _utils.consoleMessage)(debugMode, 'Setting up initial state tree');
    } else if (!action.type) {
      (0, _utils.consoleError)(debugMode, 'Action is missing type');
    }

    if (state === undefined) {
      return initialState;
    }

    var successLogs = [];
    var errorLogs = [];

    var newState = state;
    var nodes = map.get(action.type);
    if (nodes) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = nodes[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var node = _step2.value;

          var valid = node.params.map(function (p) {
            return action[p];
          }).reduce(function (prev, curr) {
            return prev && curr !== undefined;
          }, true);
          if (valid) {
            successLogs.push('Executing action ' + action.type);
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
              for (var _iterator3 = node.params[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                var param = _step3.value;

                successLogs.push('$' + param + ' = ' + action[param]);
              }
            } catch (err) {
              _didIteratorError3 = true;
              _iteratorError3 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }
              } finally {
                if (_didIteratorError3) {
                  throw _iteratorError3;
                }
              }
            }

            newState = _reduce(newState, action, node);
            break;
          } else {
            errorLogs.push('Could not execute action ' + action.type + ' with params ' + node.params);
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    } else {
      var legacyNodes = map.get(remergeLegacyKey);
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = legacyNodes[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var node = _step4.value;

          successLogs.push('Executing legacy action ' + action.type);
          newState = _reduce(newState, action, node);
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }
    }

    if (successLogs.length > 0) {
      (0, _utils.consoleGrouped)(debugMode, 'Processing action ' + action.type);
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = successLogs[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var log = _step5.value;

          (0, _utils.consoleSuccess)(debugMode, log);
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5.return) {
            _iterator5.return();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }

      (0, _utils.consoleEndGrouped)(debugMode);
    } else if (errorLogs.length > 0) {
      (0, _utils.consoleGrouped)(debugMode, 'Processing action ' + action.type, false);
      var _iteratorNormalCompletion6 = true;
      var _didIteratorError6 = false;
      var _iteratorError6 = undefined;

      try {
        for (var _iterator6 = errorLogs[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
          var log = _step6.value;

          (0, _utils.consoleSuccess)(debugMode, log);
        }
      } catch (err) {
        _didIteratorError6 = true;
        _iteratorError6 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion6 && _iterator6.return) {
            _iterator6.return();
          }
        } finally {
          if (_didIteratorError6) {
            throw _iteratorError6;
          }
        }
      }

      (0, _utils.consoleEndGrouped)(debugMode);
    } else {
      (0, _utils.consoleGrouped)(debugMode, 'Processing action ' + action.type, false);
      (0, _utils.consoleError)(debugMode, 'No available action found!');
      (0, _utils.consoleEndGrouped)(debugMode);
    }

    return newState;
  };
};

exports.default = merge;