'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isMap = isMap;
exports.debug = debug;
exports.printTree = printTree;
exports.getCollectionElement = getCollectionElement;
exports.setCollectionElement = setCollectionElement;
exports.consoleMessage = consoleMessage;
exports.consoleWarning = consoleWarning;
exports.consoleSuccess = consoleSuccess;
exports.consoleError = consoleError;
exports.consoleGrouped = consoleGrouped;
exports.consoleEndGrouped = consoleEndGrouped;

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isMap(obj) {
  return Object.getPrototypeOf(obj) === Map.prototype || obj.constructor.name === 'Map'; // Ava has a map shim which doesn't play nicely
}

function debug(map, state, action) {
  console.log('----------------------');
  console.log('Running process');
  console.log('current map');
  console.log(map);
  console.log('current state');
  console.log(state);
  console.log('current action');
  console.log(action);
  console.log('----------------------');
}

function printTree(obj) {
  console.log(_util2.default.inspect(obj, false, null));
}

function getCollectionElement(collection, key) {
  if (isMap(collection)) {
    return collection.get(key);
  } else {
    return collection[key];
  }
}

function setCollectionElement(collection, key, value) {
  if (isMap(collection)) {
    collection.set(key, value);
  } else {
    collection[key] = value;
  }
}

var colors = {
  black: "font-weight : bold; color : #000000;",
  gray: "font-weight : bold; color : #1B2B34;",
  red: "font-weight : bold; color : #EC5f67;",
  orange: "font-weight : bold; color : #F99157;",
  yellow: "font-weight : bold; color : #FAC863;",
  green: "font-weight : bold; color : #99C794;",
  teal: "font-weight : bold; color : #5FB3B3;",
  blue: "font-weight : bold; color : #6699CC;",
  purple: "font-weight : bold; color : #C594C5;",
  brown: "font-weight : bold; color : #AB7967;"
};

function _console(msg, color) {
  var func = arguments.length <= 2 || arguments[2] === undefined ? console.log : arguments[2];

  if (typeof window === 'undefined') {
    func('[remerge] ' + msg);
  } else {
    func.call(console, '%c[remerge]%c ' + msg, color, '');
  }
}

function consoleMessage(debugMode, msg) {
  if (debugMode) {
    _console(msg, colors.black);
  }
}

function consoleWarning(debugMode, msg) {
  if (debugMode) {
    _console(msg, colors.yellow);
  }
}

function consoleSuccess(debugMode, msg) {
  if (debugMode) {
    _console(msg, colors.green);
  }
}

function consoleError(debugMode, msg) {
  if (debugMode) {
    _console(msg, colors.red);
  }
}

function consoleGrouped(debugMode, msg) {
  var collapsed = arguments.length <= 2 || arguments[2] === undefined ? true : arguments[2];

  if (debugMode && console.group) {
    _console(msg, colors.black, collapsed ? console.groupCollapsed : console.group);
  } else if (debugMode) {
    _console(msg, colors.black);
  }
}

function consoleEndGrouped(debugMode) {
  if (debugMode && console.groupEnd) {
    console.groupEnd();
  }
}